# -*- coding: utf-8 -*-
"""
Created on Fri Oct 14 21:16:19 2022

@author: Administrator
"""
import threading
import socket 
import os
import time
import json
import struct
import signal
import queue
import tabulate
import hashlib
import psutil
import pandas as pd



def get_local_ip():
    """
    查询本机ip地址
    :return: ip
    """
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip
def my_handler(signum, frame):
    print('程序退出')
    os._exit(0)

signal.signal(signal.SIGINT, my_handler)


def send_msg(client,result):
    result = json.dumps(result).encode('utf-8')    
    client.sendall(result)


def wait_accept(server):
    while True:
        try:
            client, address = server.accept()
            start_thread(target = handle_connect,args=(client,address))
        except Exception as e: 
            time.sleep(1)
    server.shutdown(socket.SHUT_RDWR)
    close_con(server)
    
def start_thread(target,args):
    thp1 = threading.Thread(target=target,args=args)
    thp1.start()

def sys_show_on(data):
    connect_show_list.append(data)

def close_con(client):
    try:
        # 强制停止读写操作，确保 recv 不再阻塞
        client.shutdown(socket.SHUT_RDWR)
    except socket.error as e:
        print(f"shutdown 发生错误: {e}")
    finally:
        # 释放套接字资源
        close_con(close)
        print("连接已关闭")

def handle_connect(client,address):
     global id_code
     while True:
        # 接收客户端发过来的数据 循环为这个客户端服务多次
        recv_data = client.recv(1024)
        sys_show_on("%s:收到客户端请求连接的消息：%s" %(get_str_time(),recv_data.decode("utf-8")))
        print("%s:收到客户端%s请求连接的消息：%s" %(get_str_time(),address,recv_data.decode("utf-8")))
        dict_data = json.loads(recv_data)
        con_type = dict_data['con_type']
        con_tocken = dict_data['tocken']
        if con_tocken != tocken:
            code = '-1'
            msg = '传入的tocken:%s不匹配'%(con_tocken)
            result = {'code':code,'msg':'服务器发来消息：%s'%(msg)}
            send_msg(client,result)
        elif con_type == 'put_mode':
            # print('请求连接tx')
            result = handle_connect_put(client,address)
            C_que = queue.Queue(maxsize=0)
            C_que_push = queue.Queue(maxsize=0)
            start_thread(target = main_handle_client_que,args=(client, C_que, C_que_push))
            start_thread(target = main_push_data_que,args=(client, C_que_push))
            send_msg(client,result)
            start_thread(target = main_handle_msg_tx,args=(client,address,C_que)) 
        elif con_type == 'push_mode':
            who = dict_data['who']
            if '@' not in who:
                who = [who]
            else:
                who = who.split('@')
            pwd = dict_data['pwd']
            result = handle_connect_push(client,address,who,pwd)
            send_msg(client,result)
        elif con_type == 'file_mode':
            handle_connect_file(client,address,dict_data)
            close_con(client)
            break
        elif con_type == 'plus_mode':
            id_code = id_code + 1
            result = {'code':0,'msg':'plus mode连接成功','id_code':id_code}
            start_thread(target = main_handle_push_plus,args=(client,))
            send_msg(client,result)
        elif con_type == 'check_version':
            from tx import txl
            tx1 = txl('a',2025,'test')
            version = tx1.version
            result = {'LTtx_lastest':version}
            send_msg(client,result)
        # 如果客户端发送的数据不为空那么就是需要服务
        else:
            result = '当前连接未通过'
            send_msg(client,result)
            close_con(client)
        break



def handle_connect_file(client,address,dict_data):
    file_mode = dict_data['file_mode']
    if file_mode == 'upload_file':
        file_name = dict_data['file_name']
        file_hash = dict_data['file_hash']
        print(dict_data)
        print('当前文件已经存在')
        file = open('./file_data/%s'%(file_name+'.tmp'),'wb')
        client.sendall(b'i am ok')
        file_data = client.recv(1024)
        while file_data:
            file.write(file_data)
            file_data = client.recv(1024)
        print('文件接收完成')
        file.close()
        sys_show_on('%s文件接收完成'%(file_name))
        if os.path.isfile('./file_data/%s'%(file_name)):
            os.remove('./file_data/%s'%(file_name))
        os.rename('./file_data/%s'%(file_name + '.tmp'), './file_data/%s'%(file_name))
       
    elif file_mode == 'download_file':
        file_name = dict_data['file_name']
        hash_md5 = hashlib.md5()
        if os.path.isfile('./file_data/%s'%(file_name)):
            client.sendall('file exist'.encode())
            file = open(file_name, 'rb')
            file_data = file.read(1024)
            while file_data:
                client.sendall(file_data)
                file_data = file.read(1024)
            file.close()
            time.sleep(1)
            close_con(client)
        else:
            client.sendall('file does not exist'.encode())
    else:
        pass
    


def handle_put(var,data):
    que_put.put((var,data))
    return {'code':0,'msg':'put work down'}

que_put = queue.Queue()
def main_handle_put():
    file_path = os.path.join(os.path.dirname(__file__), 'data0.txt')
    while True:
        var,data = que_put.get()
        try:
            data = json.loads(data)
        except:
            pass
        dict_var[var] = data
        if que_put.qsize() > 50:
            while True:
                if  que_put.qsize() > 2:
                    try:
                        data = json.loads(data)
                    except:
                        pass
                    dict_var[var] = data
                else:
                    break
        with open(file_path, 'w', encoding='utf-8') as f:
                json.dump(dict_var, f, ensure_ascii=False, indent=4)


def handle_get(var):
    if var in dict_var:
        return {'value':dict_var[var]}
    else:
        return {'value':None}
def get_str_time():
    return time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())


def recv_data_from_tx(client):
    recv_bytes = client.recv(8,socket.MSG_WAITALL)
    bytes_len = struct.unpack("Q",recv_bytes)[0]
    recv_data = client.recv(bytes_len,socket.MSG_WAITALL)
    recv_data = json.loads(recv_data.decode('utf-8',errors='replace'))
    return recv_data

def recv_data_from_tx_fast(client):
    recv_bytes = client.recv(8,socket.MSG_WAITALL)
    bytes_len = struct.unpack("Q",recv_bytes)[0]
    recv_data = client.recv(bytes_len,socket.MSG_WAITALL)
    recv_data = recv_data.decode('utf-8',errors='replace')
    return recv_data

def send_data_to_client_fast(client,msg):
    msg = msg.encode('utf-8')
    data_len = len(msg)
    struct_bytes = struct.pack('Q', data_len)
    client.sendall(struct_bytes)
    client.sendall(msg)
        
def send_data_to_client(client,msg):
    msg=json.dumps(msg).encode('utf-8')
    data_len = len(msg)
    struct_bytes = struct.pack('Q', data_len)
    client.sendall(struct_bytes)
    client.sendall(msg)

def main_handle_push_plus(client):
    while True:
        recv_data = recv_data_from_tx_fast(client).rsplit(':who:',1)
        who = recv_data[1]
        if who in dict_client_push_group:
            if len(dict_client_push_group[who]['group']) == 0:
                pass
            else:
                dict_client_push_group[who]['que'].put((client,recv_data[0]))
    

def main_handle_msg_tx(client,address,C_que):
    sys_show_on(get_str_time()+':'+'客户端'+str(address)+'加入tx')
    while True:
        try:
            recv_data = recv_data_from_tx(client)
            C_que.put(recv_data)
        except Exception as e:
            close_con(close)
            print(e)
            break

def main_handle_client_que(client,C_que,C_que_push):
    while True:
        try:
            recv_data = C_que.get()
            func = recv_data['func']
            if func == 'push':
                who = recv_data['who']
                if who in dict_client_push_group:
                    if len(dict_client_push_group[who]['group']) == 0:
                        pass
                    else:
                        recv_data = recv_data['value']
                        dict_client_push_group[who]['que'].put((client,recv_data))
                else:
                    pass
            elif func == 'get':
                var = recv_data['value']
                result = handle_get(var)
                C_que_push.put(result)
            elif func == 'put':
                data = recv_data['value'][1]
                var = recv_data['value'][0]
                result = handle_put(var,data)
            elif func == 'list_append':
                data = recv_data['value'][1]
                var = recv_data['value'][0]
                result = handle_list_append(var,data)
            elif func == 'dict_change':
                value = recv_data['value'][2]
                data = recv_data['value'][1]
                var = recv_data['value'][0]
                result = handle_dict_change(var,data,value) 
            elif func == 'list_pop':
                data = recv_data['value'][1]
                var = recv_data['value'][0]
                result = handle_list_pop(var,data)
            elif func == 'list_remove':
                data = recv_data['value'][1]
                var = recv_data['value'][0]
                result = handle_list_remove(var,data)
            elif func == 'put_dataframe':
                data = recv_data['value'][1]
                var = recv_data['value'][0]
                result = handle_put_dataframe(var,data)
            elif func == 'get_dataframe':
                var = recv_data['value']
                result = handle_get_dataframe(var)
                C_que_push.put(result)
            elif func == 'get_dict_value':
                result = handle_get_dict_value(recv_data['value'][0],recv_data['value'][1])
                C_que_push.put(result)
            elif func == 'get_list_value':
                result = handle_get_list_value(recv_data['value'][0],recv_data['value'][1])
                C_que_push.put(result)
        except:
            pass


def handle_get_dict_value(var,key):
    if var in dict_var:
        if key in dict_var[var]:
            return {'value':dict_var[var][key]}
        else:
            return {'value':None}
    else:
        return {'value':None}

def handle_get_list_value(var,index):
    if var in dict_var:
        if abs(index) > len(dict_var[var]):
            return {'value':dict_var[var][index]}
        else:
            return {'value':None}
    else:
        return {'value':None}        

def handle_get_dataframe(var):
    '''
    处理get dataframe
    '''
    if var in dict_df:
        try:
            df = pd.read_csv('./dataframe_data/%s.csv'%(var))
        except:
            df = pd.DataFrame()
    else:
        df = pd.DataFrame()
    
    return {'value':json.dumps(df.to_dict(orient='records'))}

dict_df = {}
def handle_put_dataframe(var,data):
    try:
        df = pd.DataFrame(json.loads(data))
        df.to_csv('./dataframe_data/%s.csv'%(var),index=False)
        dict_df[var] = 1
        tem_dict = {'code':0,'msg':'success','value':{}}
    except Exception as e:
        tem_dict = {'code':-1,'msg':str(e)}
        print(tem_dict)
    return tem_dict


def handle_list_remove(var,data):
    global dict_var_on
    try:
        dict_var[var].remove(data)
        if dict_var_on == False:
            dict_var_on = True
    except:
        pass

def handle_list_pop(var,data):
    global dict_var_on
    try:
        data = int(data)
        dict_var[var].pop(data)
        if dict_var_on == False:
            dict_var_on = True
    except:
        pass

def handle_list_append(var,data):
    global dict_var_on
    try:
        dict_var[var].append(data)
        if dict_var_on == False:
            dict_var_on = True
    except:
        pass

def handle_dict_change(var,key,value):
    global dict_var_on
    # try:
    print(dict_var)
    # tem_dict = json.loads(dict_var[var])
    # tem_dict[key] = value
    dict_var[var][key] = value
    if dict_var_on == False:
        dict_var_on = True
    # except:
    #     pass

def main_push_data_que(client,C_que_push):
    while True:
        result = C_que_push.get()
        # print(result,'----')
        send_data_to_client(client,msg=json.dumps(result))
                
        
def handle_connect_put(con,address):
    global id_code
    id_code = id_code + 1
    dict_client[id_code] = {'con':con,'address':address}
    msg = '服务端发来信息：新连接加入成功'
    code = 0 
    result = {'code':code,'msg':msg,'id_code':id_code}
    return result

def main_heartbeat():
    '''
    客户端心跳维护子线程

    Returns
    -------
    None.

    '''
    recv_data = {1:1}
    while True:
        for who,value in dict_client_push_group.items():
            value['que'].put((1,recv_data))
        time.sleep(5)

def main_push_client(tem_que,client):
    while True:
        msg = tem_que.get()
        try:
            send_data_to_client(client, msg)
        except Exception as e:
            sys_show_on('有broadcast客户端退出')
            close_con(client)
            del dict_client_que[client]
            break
            

def handle_connect_push(con,address,who_list,pwd='520'):
    id_code = len(dict_client_push)+1
    dict_client_push[id_code] = {'con':con,'address':address}
    for who in who_list:
        if who in dict_client_push_group:
            if pwd == dict_client_push_group[who]['pwd']:
                dict_client_push_group[who]['group'].append(con)
                msg = '服务端发来信息：欢迎加入%s信道,当前总信道连接数为:%s'%(who,len(dict_client_push_group[who]['group']))
                tem_que = queue.Queue()
                dict_client_que[con] = tem_que
                threading.Thread(target = main_push_client,args=(tem_que,con)).start()
                code = 0
            else:
                msg = '服务端发来信息：当前信道%s已经存在，传入的密码:%s不正确，请重新传入密码'%(who,pwd)
                del dict_client_push[id_code]
                code = -1
        else:
            tem_que = queue.Queue()
            dict_client_que[con] = tem_que
            threading.Thread(target = main_push_client,args=(tem_que,con)).start()
            tem_que = queue.Queue(maxsize=0)
            dict_client_push_group[who] = {'group':[con],'pwd':pwd,'que':tem_que}
            thp0 = threading.Thread(target = main_push_queue,args=(tem_que,who))
            thp0.start()
            msg = '服务端发来信息：新信道%s创建成功'%(who)
            code = 0
        result = {'code':code,'msg':msg}
    return result

def main_push_queue(que,who):
    sys_show_on('新信道%s子线程启动'%(who))
    while True:
        # try:
            client,recv_data = que.get()
            group_list = dict_client_push_group[who]['group']
            if len(group_list) ==0:
                sys_show_on('%s信道退出'%(who))
                del dict_client_push_group[who]
                break
            else:
                for i in group_list:
                    if i in dict_client_que:    
                        dict_client_que[i].put(recv_data)
                    else:
                        group_list.remove(i)
     


def build_header_push(data_len):
    header = {'data_len': data_len}
    return json.dumps(header).encode('utf-8')
    

def main_test():
    while True:
        print('\n\r')
        a=input('输入要执行的python代码:\n\n')
        print('执行结果如下:\n\n')
        if a=='clear':
            os.system('clear')
        else:
            try:
                exec(a)
                print('\n')
            except Exception as e:
                print(e)

def load_dict_var():
    try:
        # 假设 data0.txt 和 LTtx_server.py 在同一目录
        config_path = os.path.join(os.path.dirname(__file__), 'data0.txt')
        
        # 尝试打开并读取 JSON 文件
        with open(config_path, 'r', encoding='utf-8') as f:
            dict_var = json.load(f)
           # print('Load dict_var success!')
    except FileNotFoundError:
        #print(f"Load Error: File not found at {config_path}")
        dict_var = {}
    except json.JSONDecodeError:
       # print("Load Error: File is not valid JSON")
        dict_var = {}
    except Exception as e:
      #  print('Load Error:', e)
        dict_var = {}
    
    return dict_var

def main_show():
    while True:
        table_data = []
        for who,value in dict_client_push_group.items():
            print(who,value['que'].qsize())
            tem_list = [who[:10],len(value['group']),value['que'].qsize(),'']
            table_data.append(tem_list)
        print('\033c', end='')

        print('#'*20,'服务器信息V5','#'*20)
        print('ip:%s'%(ip),' '*10,'port:%s'%(port),' '*10,'tocken:%s'%(tocken))
        
        print('#'*20,'当前LTtx状态','#'*20)
        print('当前TX连接数：%s                 当前TXG信道数'%(len(dict_client)),len(dict_client_push_group))
        
        print(tabulate.tabulate(table_data, headers=['信道名称', '当前连接数', '信道队列数据量','备注'], tablefmt='fancy_grid',stralign='wrap'))

        print('%sLTtx系统最新日志信息：'%(time.strftime('%Y-%m-%d %H:%M:%S',time.localtime())))
        print('#'*30)
        for i in connect_show_list:
            print(i)
            if len (connect_show_list) > 10:
                connect_show_list.remove(i)
        print('#'*30)
        
        
        time.sleep(1)

def make_dir():
    flod_list = ['file_data','dataframe_data']
    for i in flod_list:
        try:
            os.mkdir(i)
        except:
            pass

def main_save_dict_var():
    global dict_var_on, dict_var
    # 获取当前脚本所在目录，并构建文件路径
    file_path = os.path.join(os.path.dirname(__file__), 'data0.txt')

    while True:
        if dict_var_on:
            dict_var_on = False  # 重置标志
            with open(file_path, 'w', encoding='utf-8') as f:
                json.dump(dict_var, f, ensure_ascii=False, indent=4)
            print(f"dict_var saved to {file_path}")
        time.sleep(60)  # 每 60 秒检查一次

            
def main_zmq_mode(zmq_xsub_port,zmq_xpub_port):
    import zmq
    context = zmq.Context()
    # 创建并绑定发布者套接字
    frontend = context.socket(zmq.XSUB)
    frontend.setsockopt(zmq.SNDHWM, 10000000)
    frontend.setsockopt(zmq.RCVHWM, 10000000)
    frontend.bind("tcp://*:%s"%(zmq_xsub_port))

    # 创建并绑定订阅者套接字
    backend = context.socket(zmq.XPUB)
    backend.setsockopt(zmq.SNDHWM, 10000000)
    backend.setsockopt(zmq.RCVHWM, 10000000)
    backend.bind("tcp://*:%s"%(zmq_xpub_port))
    print("tcp://*:%s"%(zmq_xpub_port))
    print('zmq启动成功')
    # 启动代理
    zmq.proxy(frontend, backend)

def load_config():
    tem_dict = {}
    config_path = os.path.join(os.path.dirname(__file__), 'Config.txt')
    with open(config_path, 'r', encoding='utf-8') as f:
        data = f.read().split('\n')
        for i in data:
            if len(i) > 0 and ' ' in i and ':' in i:
                key = i.split(' ')[0].split(':')[0]
                value = i.split(' ')[0].split(':')[1]
                tem_dict[key] = value
    return tem_dict

def main():
    version = 'V7'
    import multiprocessing
    config = load_config()
    port = int(config['port'])
    tocken = config['token']
    ip = get_local_ip()
    #ZMQ的订阅端口
    zmq_xsub_port = config['zmq_port1']
    #ZMQ的发布端口
    zmq_xpub_port = config['zmq_port2']
    zmq_mode = config['zmq_mode']
    set_cpu = config['set_cpu']
    print(zmq_mode,type(zmq_mode))
    if zmq_mode == "True":
        p0 = multiprocessing.Process(target=main_zmq_mode,args=(zmq_xsub_port,zmq_xpub_port))
        p0.daemon = True
        p0.start()
        
    if set_cpu:        
        # 创建一个Process对象，代表当前进程
        p = psutil.Process(os.getpid())
        
        # 获取当前进程的CPU亲和性
        print(p.cpu_affinity())
        
        # 将当前进程的CPU亲和性设置为只运行最后一颗CPU上
        p.cpu_affinity([p.cpu_affinity()[-1]])
        
        # 验证设置是否生效
        print(p.cpu_affinity())
    
    id_code = 0
    all_que = queue.Queue(maxsize=0)

    recv_buffer_size = 1024*1024*100
    server = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    server.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, recv_buffer_size,)
    server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    server.settimeout(5)
    server.bind((ip,port))
    server.listen(5)
    print('\n\r')
    print('#'*20,'服务器信息%s'%(version),'#'*20)
    print('ip:%s'%(ip),' '*10,'port:%s'%(port),' '*10,'tocken:%s'%(tocken))
#    threading.Thread(target=main_show).start()
    threading.Thread(target = main_heartbeat).start()
    threading.Thread(target = main_save_dict_var).start()
    threading.Thread(target = main_handle_put).start()
    wait_accept(server)

#配置
connect_show_list = []
dict_var_on = False
make_dir()
dict_client_que = {}
dict_client_push = {}
dict_client_push_group = {}
dict_client = {}
dict_client_status = {}
dict_var = load_dict_var()#保存云变量

if __name__ == '__main__':
    main()


















# -*- coding: utf-8 -*-
"""
Created on Thu Sep 22 11:09:51 2022

@author: Administrator
"""
import socket
import threading
import json
import queue
import struct
import time
import os
import hashlib
import random
import datetime

class txl:
    def __init__(self,ip,port,tocken,show=True,check_version=False):
        self.ip = ip
        self.port = port
        self.tocken = tocken
        self.start_tx_on = False
        self.txg = False
        self.dict_txg = {}#存储txg对象
        self.Q = queue.Queue(maxsize=0)
        self.txg_running = False
        self.__tx = False
        self.__tx_plus = False
        self.push_count = 0
        self.timeout = 2
        self.heartbeat = 1
        self.tx_que = queue.Queue(maxsize=0)
        self.tx_que_plus = queue.Queue(maxsize=0)
        self.file_tx = None
        #ZMQ模式
        self.__ZMQ = None
        self.__ZMQ_broadcast = None
        self.__txg_heartbeat_on = True#通信系统Push模式心跳检测线程状态
        self.__txg_heartbeat_time = time.time()
        self.version = '7.0.2'
        self.__version__ = self.version
        self.check_version_on = check_version
        self.sys_print_on = show
        self.dict_TradeDay = {}
        self.check_version()
        self.__version__msg = '更新于2024-09-07'
        print('#'*20,'通信系统V%s加载成功,Have fun!'%(self.__version__),'#'*20)
        print('#'*21,'当前版本更新于%s'%(self.__version__msg),'#'*21)
        
    
    def check_version(self,):
        '''
        检查本地客户端版本是不是最新的
        '''
        if self.check_version_on:
            client = socket.socket(socket.AF_INET, socket.SOCK_STREAM) 
            client.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 8192*100)
            client.connect(('pc.zltnet.top',2099)) 
            tem_dict = {'con_type':'check_version',
                        'tocken':'LTtx',
                        }
            client.sendall(json.dumps(tem_dict).encode('utf-8'))
            server_msg = client.recv(1024).decode()
            if server_msg['LTtx_lastest'] != self.__version__:
                self.sys_print('从LTtx官网获取最新tx.py')
                self.__get_lastest_file()
                self.sys_print('最新版本获取完成，程序2秒后重启')
                time.sleep(2)
                os._exit(1)
            else:
                self.sys_print('当前版本已经是最新')
                
    def update_tx_version(self):
        '''
        更新客户端到最新版本
        
        '''
        client = socket.socket(socket.AF_INET, socket.SOCK_STREAM) 
        client.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 8192*100)
        client.connect(('pc.zltnet.top',2099)) 
        tem_dict = {'con_type':'check_version',
                    'tocken':'LTtx',
                    }
        client.sendall(json.dumps(tem_dict).encode('utf-8'))
        server_msg = json.loads(client.recv(1024).decode())
        if server_msg['LTtx_lastest'] != self.__version__:
            self.sys_print('从LTtx官网获取最新tx.py')
            self.__get_lastest_file()
            self.sys_print('最新版本获取完成，程序2秒后重启')
            time.sleep(2)
            os._exit(1)
        else:
            self.sys_print('当前版本已经是最新')    
    
    def sys_print(self,data):
        if self.sys_print_on:
            print(self.get_nowtime(),'LTtx[info]>>>>:',data)

    def send_file(self,file_name,show_on=True):
        '''
        将本地文件上传至数据中心

        Parameters
        ----------
        file_name : TYPE
            文件路径，通常为./data/file.txt.
        show_on : TYPE, optional
            是否Print进度，默认开启. The default is True.

        Returns
        -------
        code int.
        返回0表示成功，其他表示错误，参见msg
        msg string
        提示信息

        '''
        if os.path.isfile(file_name):
            if show_on:
                print('识别到文件存在')
                print(file_name)
            client = socket.socket(socket.AF_INET, socket.SOCK_STREAM) 
            client.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 8192*100)
            client.connect((self.ip,self.port)) 
            hash_md5 = hashlib.md5()
            with open(file_name, "rb") as f:
                for chunk in iter(lambda: f.read(4096), b""):
                    hash_md5.update(chunk)
            file_hash = hash_md5.hexdigest()
            tem_dict = {'con_type':'file_mode',
                        'tocken':self.tocken,
                        'file_name':file_name.rsplit('/',1)[-1],
                        'file_hash':file_hash,
                        'file_mode':'upload_file',
                        }
            client.sendall(json.dumps(tem_dict).encode('utf-8'))
            server_msg = client.recv(1024).decode()
            if server_msg == 'i am ok':
                t1 = time.time()
                file = open(file_name, 'rb')
                file_data = file.read(1024)
                while file_data:
                    client.send(file_data)
                    file_data = file.read(1024)
                file.close()
                # client.send(b'file send finish')
                # client.recv(1024).decode()
                client.close()
                if show_on:
                    print('file send done! usetime:%ss'%(round(time.time()-t1,6)))
            else:
                print('服务端拒绝了本次文件传输请求')
        else:
            print('文件不存在,请重新传入,当前收到的文件名:')
            print(file_name)
    
    def __get_lastest_file(self,file_name='tx.py',file_path='./',show_on=False):
        '''
        从服务端下载文件

        Parameters
        ----------
        file_name : TYPE
            要下载的文件名.
        file_path : TYPE, optional
            文件保存路径，不存在的路径将会被创建. The default is './'.
        show_on : TYPE, optional
            是否Print进度，默认开启. The default is True.
        Returns
        -------
        code int.
        返回0表示成功，其他表示错误，参见msg
        msg string
        提示信息

        '''
        if file_path[-1] != '/':
            file_path = file_path + '/'
        if os.path.isdir(file_path):
            pass
        else:
            print('文件路径不存在，自动创建该路径')
            try:
                os.mkdir(file_path)
            except Exception as e:
                print(e)
                raise TypeError('文件路径自动创建失败,失败原因：%s'%(e))
        client = socket.socket(socket.AF_INET, socket.SOCK_STREAM) 
        client.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 8192*100)
        client.connect(('pc.zltnet.top',2099)) 
        tem_dict = {'con_type':'file_mode',
                    'tocken':'LTtx',
                    'file_name':file_name,
                    'file_mode':'download_file',
                    }
        client.sendall(json.dumps(tem_dict).encode('utf-8'))
        server_msg = client.recv(1024).decode()
        if server_msg == 'file exist':
            t1 = time.time()
            file = open(file_path+file_name+'.tmp','wb')
            file_data = client.recv(1024)
            while file_data:
                file.write(file_data)
                file_data = client.recv(1024)
               
            file.close()
            if os.path.isfile(file_path+file_name):
                os.remove(file_path+file_name)
            os.rename(file_path+file_name+ '.tmp', file_path+file_name)
            code = 0
            msg = '%s文件接收完成,用时%ss'%(file_name,round(time.time()-t1,6))
            if show_on:
                print(code,msg)
            return code,msg
        else:
            code = -1
            msg = '服务端该文件不存在,请先上传'
            print(code,msg)
            return code,msg
    
    
    def recv_file(self,file_name,file_path='./',show_on=False):
        '''
        从服务端下载文件

        Parameters
        ----------
        file_name : TYPE
            要下载的文件名.
        file_path : TYPE, optional
            文件保存路径，不存在的路径将会被创建. The default is './'.
        show_on : TYPE, optional
            是否Print进度，默认开启. The default is True.
        Returns
        -------
        code int.
        返回0表示成功，其他表示错误，参见msg
        msg string
        提示信息

        '''
        if file_path[-1] != '/':
            file_path = file_path + '/'
        if os.path.isdir(file_path):
            pass
        else:
            print('文件路径不存在，自动创建该路径')
            try:
                os.mkdir(file_path)
            except Exception as e:
                print(e)
                raise TypeError('文件路径自动创建失败,失败原因：%s'%(e))
        client = socket.socket(socket.AF_INET, socket.SOCK_STREAM) 
        client.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 8192*100)
        client.connect((self.ip,self.port)) 
        tem_dict = {'con_type':'file_mode',
                    'tocken':self.tocken,
                    'file_name':file_name,
                    'file_mode':'download_file',
                    }
        client.sendall(json.dumps(tem_dict).encode('utf-8'))
        server_msg = client.recv(1024).decode()
        if server_msg == 'file exist':
            t1 = time.time()
            file = open(file_path+file_name+'.tmp','wb')
            file_data = client.recv(1024)
            while file_data:
                file.write(file_data)
                file_data = client.recv(1024)
            file.close()
            code = 0
            msg = ('%s文件接收完成,文件MD5检验通过'%(file_name))
            if os.path.isfile(file_path+file_name):
                os.remove(file_path+file_name)
            os.rename(file_path+file_name+ '.tmp', file_path+file_name)
            code = 0
            msg = '%s文件接收完成,,用时%ss'%(file_name,round(time.time()-t1,6))
            if show_on:
                print(code,msg)
            return code,msg
        else:
            code = -1
            msg = '服务端该文件不存在,请先上传'
            print(code,msg)
            return code,msg
    
    def start_tx(self):
        while True:
            try:
                print(self.get_nowtime(),'start_tx正在连接(%s,%s)LTtx服务器,请稍后'%(self.ip,self.port))
                if self.__tx == False:
                    client = socket.socket(socket.AF_INET, socket.SOCK_STREAM) 
                    client.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 8192*100)
                    client.connect((self.ip,self.port))
                    self.__tx = client
                    tem_dict = {'con_type':'put_mode','tocken':self.tocken}
                    self.__tx.sendall(json.dumps(tem_dict).encode('utf-8'))
                    if self.recv_msg_start_tx(client):
                        t0 = threading.Thread(target=self.main_tx_que)
                        t0.start()
                        thp0 = threading.Thread(target = self.start_tx_hearbeat)
                        thp0.start()
                        break
                    else:
                        self.close_connection(self.__tx)
                        self.__tx = False 
                else:
                    code = -1
                    msg = '请勿重复连接tx'
                    result = {'code':code,'msg':msg}
                    print(result)
                    return result
            except Exception as e:
                # if type(e) == ConnectionRefusedError:
                print(self.get_nowtime(),'服务端未启动,将在1秒后继续尝试start_tx')
                print(e)
                time.sleep(1)
    
    def start_plus(self,):
        while True:
            try:
                print(self.get_nowtime(),'start_flash正在连接(%s,%s)LTtx服务器,请稍后'%(self.ip,self.port))
                if self.__tx_plus == False:
                    client = socket.socket(socket.AF_INET, socket.SOCK_STREAM) 
                    client.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 8192*100)
                    client.connect((self.ip,self.port))
                    self.__tx_plus = client
                    tem_dict = {'con_type':'plus_mode','tocken':self.tocken}
                    self.__tx_plus.sendall(json.dumps(tem_dict).encode('utf-8'))
                    if self.recv_msg_start_tx(client):
                        t0 = threading.Thread(target=self.main_tx_que_plus)
                        t0.start()
                        thp0 = threading.Thread(target = self.start_tx_hearbeat_plus)
                        thp0.start()
                        break
                    else:
                        self.close_connection(self.__tx_plus)
                        self.__tx_plus = False 
                else:
                    code = -1
                    msg = '请勿重复连接'
                    result = {'code':code,'msg':msg}
                    print(result)
                    return result
                
            except Exception as e:
                if type(e) == ConnectionRefusedError:
                    print(self.get_nowtime(),'服务端未启动,将在1秒后继续尝试start_plus')
                print(self.get_nowtime(),e)
                time.sleep(1)
        # pass
    
    def start_txg(self,channel_list,pwd=''):
        while True:
            if not self.__tx:
                self.start_tx()
            try:
                if self.txg==False:
                    self.channel_list = channel_list
                    txg_count = 0
                    for who in self.channel_list.split('@'):
                        if len(who) == 0:
                            continue
                        client = socket.socket(socket.AF_INET, socket.SOCK_STREAM) 
                        client.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 1024*1024*100)
                        self.txg_running = True
                        client.connect((self.ip,self.port,))
                        self.channel_pwd = pwd
                        tem_dict = {'con_type':'push_mode','who':who,'pwd':pwd,'tocken':self.tocken}
                        client.send(json.dumps(tem_dict).encode('utf-8'))
                        if self.recv_msg_start_txg(client) ==True:      
                            self.txg = client
                            self.__start_thread(target = self.recv_msg_broadcast, args = (client,))  
                        else:
                            self.close_connection(self.txg)
                            self.txg = False
                        time.sleep(1)
                    if self.txg != False:
                        print(self.get_nowtime(),'start_txg连接成功')
                        break
                else:
                    code = -1
                    msg = '请勿重复连接txg'
                    result = {'code':code,'msg':msg}
                    print(result)
                    return result
            except Exception as e:
                if type(e) == ConnectionRefusedError:
                    print(self.get_nowtime(),'服务端未启动,将在1秒后继续尝试start_txg')
                    time.sleep(1)
                print(e)
    
    def add_txg(self,channel_name,pwd=''):
        '''
        增加txg的订阅，同txg,用@符号连接多个信道
        如 'test@test1'
        '''
        if '@' in channel_name:
            for channel in channel_name.split('@'):
                self.dict_txg[channel] = {}
                self.dict_txg[channel]['con'] = 1
        else:
            pass
    
    def cancel_txg(self,channel_name):
        '''
        取消txg的订阅，同txg,用@符号连接多个信道
        '''
        if '@' in channel_name:
            for channel in channel_name.split('@'):
                if channel in self.dict_txg:
                    self.close_connection(self.dict_txg[channel]['con'])
                    del self.dict_txg[channel]
        else:
            pass
    
    
    def start_MQ(self,pub_port=5555):
        import zmq
        '''
        开启ZMQ模式，该模式是将ZMQ与LTtx进行功能上的整合，当前版本的断线重连机制完全依赖于ZMQ自身的
        断线重连机制

        Parameters
        ----------
        pub_port : int
            传入服务端的ZMQ发布端口，通常为5555.

        Returns
        -------
        None.

        '''
        if not self.__ZMQ:
            context = zmq.Context()
            
            # 创建发布者套接字并连接到代理
            socket = context.socket(zmq.PUB)
            socket.setsockopt(zmq.RCVHWM, 1000000)
            socket.setsockopt(zmq.SNDHWM, 1000000)
            socket.connect('tcp://%s:%s'%(self.ip,pub_port))
            socket.send_string('test dsfadsafdsafsda')
            self.__ZMQ = socket
            print(self.get_nowtime(),'zmq connect ok')
        else:
            code = -1
            msg = 'start_MQ已经连接,请不要重复连接'
            print(self.get_nowtime(),msg)
    
    def start_MQ_broadcast(self,channel_list,sub_port=5556):
        '''
        开启ZMQ的订阅模式，同start_MQ

        Parameters
        ----------
        channel_list : string
            和start_txg()类似，传入要订阅的信道列表，用@进行分割.
        sub_port : int, optional
            传入服务端的ZMQ订阅端口，通常为5556

        Returns
        -------
        None.

        '''
        import zmq
        if not self.__ZMQ_broadcast:
            context = zmq.Context()

            # 创建订阅者套接字并连接到代理
            socket = context.socket(zmq.SUB)
            socket.setsockopt(zmq.RCVHWM, 1000000)
            socket.setsockopt(zmq.SNDHWM, 1000000)
            socket.connect("tcp://%s:%s"%(self.ip,sub_port))
            self.zmq_channel_list = channel_list.split('@')
            for channel in self.zmq_channel_list:
                if len(channel) > 0:
                    # 订阅特定主题
                    socket.setsockopt_string(zmq.SUBSCRIBE, channel)
            self.__ZMQ_broadcast = socket
            threading.Thread(target = self.main_recv_msg_from_zmq).start()
        else:
            code = -1
            msg = 'MQ_broadcast已经连接,请不要重复连接'
            print(self.get_nowtime(),msg)
    
    def main_recv_msg_from_zmq(self):
        print(self.get_nowtime(),'开始从ZMQ中接收数据')
        while True:
            data = self.__ZMQ_broadcast.recv_string().split('|',1)
            if data[0] in self.zmq_channel_list:
                self.Q.put(data[1])

    def __main_txg_heartbeat(self):
        if self.__txg_heartbeat_on:
            self.__txg_heartbeat_on = False
            while True:
                now_time = time.time()
                if now_time - self.__txg_heartbeat_time > 60:
                    print(self.get_nowtime(),'识别到txg因为网络不稳定导致断线，即将重连txg')
                    self.close_connection(self.txg)
                    self.start_txg(self.channel_list)
                time.sleep(10)
        else:
            print(self.get_nowtime(),'当前txg的心跳检测子线程已经启动,不再重新启动子线程监控心跳')
    
    def start_tx_hearbeat(self):
        print('start_tx的heartbeat子线程启动')
        while 1:
            if self.__tx:
                self.push('heartbeat','1','heartbeat0')
                time.sleep(self.timeout)
            else:
                break

    def start_tx_hearbeat_plus(self):
        print('start_tx_plus的heartbeat子线程启动')
        while 1:
            if self.__tx_plus:
                self.push_plus('heartbeat','t','test22')
                time.sleep(2)
            else:
                break
            
    def __start_thread(self,target,args):
        thp1 = threading.Thread(target=target,args=args)
        # thp1.setDaemon(True)
        thp1.start()        
      
    
    def recv_msg_start_tx(self,client):
        
        data = client.recv(1024)
        dict_data = json.loads(data)
        code = dict_data['code']
        self.id_code = str(dict_data['id_code'])+'@'
        if code == 0:
            return True
        else:
            return False
    
    def recv_msg_start_txg(self,client):
        data = client.recv(1024)
        dict_data = json.loads(data)
        code = dict_data['code']
        if code == 0:
            return True
        else:
            return False
            
    def recv_data_from_tx(self,client):
        recv_bytes = client.recv(8,socket.MSG_WAITALL)
        bytes_len = struct.unpack("Q",recv_bytes)[0]
        recv_data = client.recv(bytes_len,socket.MSG_WAITALL).decode('utf-8',errors='replace')
        recv_data = json.loads(recv_data)
        return recv_data
    
    def get(self,key):
        '''
        获取云变量'key'对应的值，如果不存在则返回None
        '''
        if self.__tx == False:
            code = -1
            msg = '当前tx未连接,请先执行start_tx()'
            result = {'code':code,'msg':msg}
            return result
        else:
            send_data = {'func':'get','value':key}
            msg = json.dumps(send_data)
            self.send_data(self.__tx,msg)
            result = self.recv_data_from_tx(self.__tx)
            result = json.loads(result)
            if 'value' in result:
                result = result['value']
                self.heartbeat = 0
            return result
    
    def get_dict_value(self,var,key):
        '''
        获取云变量字典“key"中对应的key值，如果不存在则返回None
        '''
        if self.__tx == False:
            code = -1
            msg = '当前tx未连接,请先执行start_tx()'
            result = {'code':code,'msg':msg}
            return result
        else:
            send_data = {'func':'get_dict_value','value':(var,key)}
            msg = json.dumps(send_data)
            self.send_data(self.__tx,msg)
            result = self.recv_data_from_tx(self.__tx)
            result = json.loads(result)
            if 'value' in result:
                result = result['value']
                self.heartbeat = 0
            return result

    def get_list_value(self,key:str,index:int):
        '''
        获取云变量列表"key"中对应下标的为index的值，index传入整型，和list使用方法一致，如果不存在则返回为None。
        '''
        if self.__tx == False:
            code = -1
            msg = '当前tx未连接,请先执行start_tx()'
            result = {'code':code,'msg':msg}
            return result
        else:
            send_data = {'func':'get_list_value','value':(key,index)}
            msg = json.dumps(send_data)
            self.send_data(self.__tx,msg)
            result = self.recv_data_from_tx(self.__tx)
            result = json.loads(result)
            if 'value' in result:
                result = result['value']
                self.heartbeat = 0
            return result


    def get_df(self,key):
        import pandas as pd
        if self.__tx == False:
            code = -1
            msg = '当前tx未连接,请先执行start_tx()'
            result = {'code':code,'msg':msg}
            return result
        else:
            send_data = {'func':'get_dataframe','value':key}
            msg = json.dumps(send_data)
            self.send_data(self.__tx,msg)
            result = self.recv_data_from_tx(self.__tx)
            result = json.loads(result)
            if 'value' in result:
                result = result['value']
                result = pd.DataFrame(json.loads(result))
                self.heartbeat = 0
            return result


    def put(self,key,data):
        try:
            if self.__tx == False:
                code = -1
                msg = '当前tx未连接,请先执行start_tx()'
                result = {'code':code,'msg':msg}
                return result
            else:
                send_data = {'func':'put','value':(key,data)}
                msg = json.dumps(send_data)
                self.send_data(self.__tx,msg)
        except:
            print('Err:和服务端失去连接，即将重连')
            self.__tx = False
            self.start_tx()
    
    def put_df(self,key,df):
        try:
            if self.__tx == False:
                code = -1
                msg = '当前tx未连接,请先执行start_tx()'
                result = {'code':code,'msg':msg}
                return result
            else:
                send_data = {'func':'put_dataframe','value':(key,json.dumps(df.to_dict(orient='records')))}
                msg = json.dumps(send_data)
                self.send_data(self.__tx,msg)
        except:
            print('Err:和服务端失去连接，即将重连')
            self.__tx = False
            self.start_tx()
    


    def list_append(self,var,data):
        '''
        对云端变量类型为列表的var进行列表append操作，相当于本地列表的基础操作，无返回值，默认成功

        Parameters
        ----------
        key : TYPE
            DESCRIPTION.
        data : TYPE
            DESCRIPTION.

        Returns
        -------
        result : TYPE
            DESCRIPTION.

        '''
        try:
            if self.__tx == False:
                code = -1
                msg = '当前tx未连接,请先执行start_tx()'
                result = {'code':code,'msg':msg}
                return result
            else:
                send_data = {'func':'list_append','value':(var,data)}
                msg = json.dumps(send_data)
                self.send_data(self.__tx,msg)
        except:
            print('Err:和服务端失去连接，即将重连')
            self.__tx = False
            self.start_tx()
    
    def list_remove(self,var,data):
        '''
        对云端变量类型为列表的var进行列表remove操作，相当于本地列表的基础操作，无返回值，默认成功

        Parameters
        ----------
        key : TYPE
            DESCRIPTION.
        data : TYPE
            DESCRIPTION.

        Returns
        -------
        result : TYPE
            DESCRIPTION.

        '''
        try:
            if self.__tx == False:
                code = -1
                msg = '当前tx未连接,请先执行start_tx()'
                result = {'code':code,'msg':msg}
                return result
            else:
                send_data = {'func':'list_remove','value':(var,data)}
                msg = json.dumps(send_data)
                self.send_data(self.__tx,msg)
        except:
            print('Err:和服务端失去连接，即将重连')
            self.__tx = False
            self.start_tx()
    
    def list_pop(self,var,data):
        '''
        对云端变量类型为列表的var进行列表pop操作，相当于本地列表的基础操作，无返回值，默认成功

        Parameters
        ----------
        key : TYPE
            DESCRIPTION.
        data : TYPE
            DESCRIPTION.

        Returns
        -------
        result : TYPE
            DESCRIPTION.

        '''
        try:
            if self.__tx == False:
                code = -1
                msg = '当前tx未连接,请先执行start_tx()'
                result = {'code':code,'msg':msg}
                return result
            else:
                send_data = {'func':'list_pop','value':(var,data)}
                msg = json.dumps(send_data)
                self.send_data(self.__tx,msg)
        except:
            print('Err:和服务端失去连接，即将重连')
            self.__tx = False
            self.start_tx()
    
    def dict_change(self,var,key,value):
        '''
        对云端变量为var的字典进行字典操作，同基础的字典操作，无返回值，默认成功

        Parameters
        ----------
        var : TYPE
            DESCRIPTION.
        key : TYPE
            DESCRIPTION.
        value : TYPE
            DESCRIPTION.

        Returns
        -------
        result : TYPE
            DESCRIPTION.

        '''
        try:
            if self.__tx == False:
                code = -1
                msg = '当前tx未连接,请先执行start_tx()'
                result = {'code':code,'msg':msg}
                return result
            else:
                send_data = {'func':'dict_change','value':(var,key,value)}
                msg = json.dumps(send_data)
                self.send_data(self.__tx,msg)
        except:
            print('Err:和服务端失去连接，即将重连')
            self.__tx = False
            self.start_tx()
    
    def push(self,key,data,who=None):
        try:
            if self.__tx == False:
                code = -1
                msg = '当前tx未连接,请先执行start_tx()'
                result = {'code':code,'msg':msg}
                return result
            else:
                send_data = {'func':'push','value':'%s|%s'%(key,data),'who':who}
                msg = json.dumps(send_data)
                self.send_data(self.__tx,msg)
                
        except Exception as e:
            print(e)
            raise ConnectionAbortedError('与服务器连接断开,push函数出错,请注意你传入的数据类型必须为字符串,请查看上方的报错内容')
            print('chucuole')
    
    def push_plus(self,key,data,who=''):
        '''
        超级push函数，会比push函数更快，效率更高，确保你传入的参数均为字符串，否则推送不成功

        Parameters
        ----------
        key : TYPE
            DESCRIPTION.
        data : TYPE
            DESCRIPTION.
        who : TYPE, optional
            DESCRIPTION. The default is None.

        Raises
        ------
        ConnectionAbortedError
            DESCRIPTION.

        Returns
        -------
        result : TYPE
            DESCRIPTION.

        '''
        try:
            if self.__tx_plus == False:
                code = -1
                msg = '当前tx_plus未连接,请先执行start_tx_plus()'
                result = {'code':code,'msg':msg}
                return result
            else:
                msg = key+'|'+data+':who:'+who
                self.send_data_plus(self.__tx_plus,msg)    
        except Exception as e:
            print(e)
            print('chucuole')
            raise ConnectionAbortedError('与服务器连接断开,push_plus函数出错,请确保你传入的参数均为字符串，请查看上方的报错内容')
         
    def MQ_push(self,var,data,who):
        '''
        采用ZMQ模式进行push，速度更快，一百万次推送耗时在1.8秒左右，但对于数据安全性没有保障，适合大通量的行情推送，在测试时发现有数据不能
        完全到达的情况，请自行做好数据校验机制

        Parameters
        ----------
        var : TYPE
            DESCRIPTION.
        data : TYPE
            DESCRIPTION.
        who : TYPE
            DESCRIPTION.

        Returns
        -------
        None.

        '''
        if self.__ZMQ:
            self.__ZMQ.send_string('%s|%s|%s'%(who,var,data))
        else:
            return (-1,'请先执行start_MQ()')
    
    def recv_msg_put(self,client):
        data = client.recv(1024)
        dict_data = json.loads(data)
        return dict_data
    
    def send_data(self,client, msg):
        self.tx_que.put(msg)
    
    def send_data_plus(self,client, msg):
        self.tx_que_plus.put(msg)
    
    def main_tx_que(self):
        while True:
            try:
                old_msg = self.tx_que.get()
                msg=old_msg.encode('utf-8')
                data_len = len(msg)
                struct_bytes = struct.pack('Q', data_len)
                self.__tx.sendall(struct_bytes)
                self.__tx.sendall(msg)
            except:
                self.__tx = False
                self.start_tx()
                if self.txg_running:
                    print('开始重连txg')
                    if self.txg:
                        self.close_connection(self.txg)
                        self.txg = False
                        self.start_txg(self.channel_list)
                        self.tx_que.put(old_msg)
                print('我退出了')
                break
            
    def main_tx_que_plus(self):
        while True:
            try:
                old_msg = self.tx_que_plus.get()
                msg=old_msg.encode('utf-8')
                data_len = len(msg)
                struct_bytes = struct.pack('Q', data_len)
                self.__tx_plus.sendall(struct_bytes)
                self.__tx_plus.sendall(msg)
            except:
                self.__tx_plus = False
                self.start_plus()
                self.tx_que_plus.put(old_msg)
                break
    
    def recv_msg_broadcast(self,client):
        while True:
            try:
                if self.txg ==False:
                    mode = 2
                    msg = '识别到通信断开，结束收取broadcast信息'
                    result = {'mode':mode,'msg':msg}
                    print(result)
                    self.close_connection(client)
                    break
                else:
                    recv_data = self.recv_data_from_tx(client)
                    if '|' in recv_data:
                        self.Q.put(recv_data)
                    else:
                        self.__txg_heartbeat_time = time.time()
            except Exception as e:
                print(self.get_nowtime(),'broadcast_error:',e)
                if self.txg != False:
                    self.close_connection(self.txg)
                    self.txg = False
                    self.start_txg(self.channel_list,pwd=self.channel_pwd)
                    break
                    time.sleep(1)
                else:
                    break
    
    def create_channel(self,num=30):
        '''
        创建一个随机信道，默认长度为30位字符串

        Parameters
        ----------
        num : int
            要创建的随机信道长度，默认为30位

        Returns
        -------
        None.

        '''        
        s='abcdefghijklmnopqrstuvwxz12345678901'
        str1 = ''
        for i in range(num):
            str1 = str1 + s[random.randint(0,35)]
        return str1 
    
    def close_connection(self,client):
        try:
            # 强制停止读写操作，确保 recv 不再阻塞
            client.shutdown(socket.SHUT_RDWR)
        except socket.error as e:
            print(f"shutdown 发生错误: {e}")
        finally:
            # 释放套接字资源
            client.close()
            print("连接已关闭")



    def close_tx(self):
        if self.__tx == False:
            code = -1
            msg = '当前tx未连接'
        else:
            self.close_connection(self.__tx)
            self.__tx = False
            code = 0
            msg = 'tx关闭成功'   
        result = {'code':code,'msg':msg}
        return result
    
    def close_tx_plus(self,):
        '''
        关闭push_plus功能

        Returns
        -------
        None.

        '''
        if self.__tx_plus == False:
            code = -1
            msg = '当前tx_plus功能未开启'
            result = {'code':code,'msg':msg}
            print(result)
            return result
        else:
            self.close_connection(self.__tx_plus)
            self.__tx_plus = False
            
            code = 0
            msg = 'tx_plus关闭成功'
            result = {'code':code,'msg':msg}
            print(result)
            return result
    
    def close_txg(self):
        if self.txg == False:
            code = -1
            msg = '当前txg未连接'
            result = {'code':code,'msg':msg}
            print(result)
            return result
        else:            
            self.close_connection(self.txg)
            self.txg = False            
            code = 0
            msg = 'txg关闭成功'
        result = {'code':code,'msg':msg}
        print(result)
        return result
    
    def get_nowdate(self,):
        '''
        返回当前的日期，格式为2024-04-04
        '''
        return time.strftime("%Y-%m-%d")

    def get_nowmin(self,):
        '''
        返回当前的时间，格式为19:00:00
        ''' 
        return time.strftime('%H:%M:%S')

    def get_nowtime(self,):
        '''
        返回当前时间，格式为2023-11-15 20:02:01

        Returns
        -------
        TYPE
            DESCRIPTION.

        '''
        return time.strftime('%Y-%m-%d %H:%M:%S',time.localtime())
    
    def get_timestamp(self):
        '''
        返回当前数字时间戳

        Returns
        -------
        None.

        '''
        return time.time()
    
    def DatestrtingToInt(self,time_str):
        '''
        把2023-10-15 20:02:01转为1697371321

        Parameters
        ----------
        time_str : string
            2023-10-15 20:02:01

        Returns
        -------
        timestamp : int
            返回整形时间戳1697371321.

        '''
        dt = datetime.datetime.strptime(time_str, '%Y-%m-%d %H:%M:%S')
        timestamp = int(dt.timestamp())
        return timestamp
    def IntTimeToString(self,ct):
        '''
        把
        conv_time(1697371321) --> '2023-10-15 20:02:01'
        '''
        local_time = time.localtime(ct)
        data_head = time.strftime('%Y-%m-%d %H:%M:%S', local_time)
        return data_head

    def calc_datetime_day(self,str1,str2):
        '''
        计算2023-10-15 20：24：55和2023-10-14 20：24：55之间的天数

        Parameters
        ----------
        str1 : TYPE
            DESCRIPTION.
        str2 : TYPE
            DESCRIPTION.

        Returns
        -------
        TYPE
            DESCRIPTION.

        '''
        return (datetime.datetime.strptime(str1, '%Y-%m-%d %H:%M:%S') - datetime.datetime.strptime(str2, '%Y-%m-%d %H:%M:%S')).days
    
    def calc_datetime_seconds(self,str1,str2):
        '''计算秒差'''
        return (datetime.datetime.strptime(str1, '%Y-%m-%d %H:%M:%S') - datetime.datetime.strptime(str2, '%Y-%m-%d %H:%M:%S')).seconds
    
    def calc_datetime_minutes(self,str1,str2):
        '''计算分钟差'''
        return (datetime.datetime.strptime(str1, '%Y-%m-%d %H:%M:%S') - datetime.datetime.strptime(str2, '%Y-%m-%d %H:%M:%S')).seconds // 60
    
    def calc_zdf(N1,N2):
        '''
        计算两个值的涨跌百分比，通常用在计算收益率中
        '''
        return (N1-N2)/N2 * 100

    def get_day_before(self,datestr,N):
        '''向前取N天,传入2023-10-15的格式'''
        date = datetime.datetime.strptime(datestr, '%Y-%m-%d')
        date_before = date - datetime.timedelta(days=N)
        return str(date_before.date())

    def get_day_next(self,datestr,N):
        '''
        向前取N天，传入2023-10-14的格式
        '''
        date = datetime.datetime.strptime(datestr, '%Y-%m-%d')
        date_before = date + datetime.timedelta(days=N)
        return str(date_before.date())
    
    def get_day_cha(self,datestr,N):
        '''
        返回N天后的日期，注意N的正负号，传入"2024-04-16 15:00:00"或者"15:00:00"
        返回值为字符串,如果N为3，返回"2024-04-19 15:00:00"
        '''
        if ' ' in datestr:
            date = datetime.datetime.strptime(datestr, '%Y-%m-%d %H:%M:%S')
        else:
            date = datetime.datetime.strptime(datestr, '%H:%M:%S')
        if N > 0:
            date_before = date + datetime.timedelta(days=N)
        else:
            date_before = date - datetime.timedelta(days=-N)
        if ' ' in datestr:
            result = str(date_before)
        else:
            result = str(date_before).split(' ')[1]
        return result


    def get_min_cha(self,datestr,N):
        '''
        返回分钟差后的日期，注意N的正负号，传入"2024-04-16 15:00:00"或者"15:00:00"
        返回值为字符串
        '''
        if ' ' in datestr:
            date = datetime.datetime.strptime(datestr, '%Y-%m-%d %H:%M:%S')
        else:
            date = datetime.datetime.strptime(datestr, '%H:%M:%S')
        if N > 0:
            date_before = date + datetime.timedelta(minutes=N)
        else:
            date_before = date - datetime.timedelta(minutes=-N)
        if ' ' in datestr:
            result = str(date_before)
        else:
            result = str(date_before).split(' ')[1]
        return result
    
    def get_second_cha(self,datestr,N):
        '''
        返回秒差后的日期，注意N的正负号，传入"2024-04-16 15:00:00"或者"15:00:00"
        返回值为字符串
        '''
        if ' ' in datestr:
            date = datetime.datetime.strptime(datestr, '%Y-%m-%d %H:%M:%S')
        else:
            date = datetime.datetime.strptime(datestr, '%H:%M:%S')
        if N > 0:
            date_before = date + datetime.timedelta(seconds=N)
        else:
            date_before = date - datetime.timedelta(seconds=-N)
        if ' ' in datestr:
            result = str(date_before)
        else:
            result = str(date_before).split(' ')[1]
        return result
    
    def judge_is_TradeDay(self,datestr=None):
        '''
        判断datestr是不是A股交易日,需要先安装pandas_market_calendars库的支持
        如果没有传入日期，则返回判断当天是不是交易日

        Parameters
        ----------
        datestring : TYPE
            DESCRIPTION.

        Returns
        bool
        -------
        如果时交易日则返回为True,否则为False.

        '''
        if not datestr:
            datestr = time.strftime("%Y-%m-%d")
        if datestr in self.dict_TradeDay:
            return self.dict_TradeDay[datestr]
        
        import pandas_market_calendars as mcal
        sse = mcal.get_calendar('SSE')#上海证券交易所日历
        td_df = sse.schedule(start_date=datestr, end_date=datestr)
        if len(td_df) > 0:
            result = True
        else:
            result = False
        self.dict_TradeDay[datestr] = result
        return result

    def get_last_TradeDay(self,):
        '''
        返回中国A股市场最近的一个交易日        
        '''
        import pandas_market_calendars as mcal
        sse = mcal.get_calendar('SSE')#上海证券交易所日历
        now_date = time.strftime("%Y-%m-%d")
        last_date = self.get_day_before(now_date,30)
        td_df = sse.schedule(start_date=last_date, end_date=now_date)        
        result = str(td_df.index[-1]).split(' ')[0]
        return result
        
    

    def get_local_ip(self,):
        '''返回局域网IPV4地址'''
        import socket
        try:
            # 创建一个socket对象
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            # 不需要真的连接，所以使用一个不存在的地址
            s.connect(("10.255.255.255", 1))
            IP = s.getsockname()[0]
        except Exception:
            IP = "127.0.0.1"  # 如果出现异常，则返回回环地址
        finally:
            s.close()  # 关闭socket
        return IP
    
    def get_public_ip(self,):
        '''
        返回公网IPV4地址
        '''
        import requests
        response = requests.get('https://httpbin.org/ip')
        ip = response.json().get('origin')
        return ip

    def judge_Time_between(str1,str2):
        '''
        判断当前时间是不是介于str1和str2之间
        str1格式: '15:00:05'
        str2格式: '16:00:00'
        
        '''
        return str1 < time.strftime('%H:%M:%S') < str2
    
    def cancel_txg(self,channel):
        '''
        取消已订阅的频道,传入对应的channel
        
        '''
        pass

    def add_txg(self,channel_list:list,
                
                ):
        '''
        新增频道订阅，传入对应的channel
        新增成功后将会接收对应的信息
        '''

    def save_json(self,file_name,obj):
        with open(file_name,'w',encoding='utf-8') as f:
            json.dump(obj,f,ensure_ascii=False,indent=2)
            

    def load_json(self,file_name):
        with open(file_name,'r',encoding='utf-8') as f :
            data = json.load(f)
        return data
    
    def save_json_fix(self,file_name,dict_obj):
        '''
        保存json，会自动将不符合的json数据转为正常的，如numpy.float64转为正常的float64，
        传入的对象为字典

        待开发
        '''
    
    def mkdir(self,file_name):
        '''
        创建一个目录，如果已经存在则会跳过
        '''
        try:
            os.mkdir(file_name)
        except:
            pass
    
    @staticmethod
    def proxy(ip, port, user='', passwd='', proxy_type='http'):
        '''
        在程序内部实现http或者socks5代理，默认为http，如果user和passwd没有传入则默认为空
        '''
        from functools import wraps
        import socks
        def decorator(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                # 构建代理 URL
                if user and passwd:
                    proxy_url = f"{proxy_type}://{user}:{passwd}@{ip}:{port}"
                else:
                    proxy_url = f"{proxy_type}://{ip}:{port}"

                # 设置代理
                proxies = {
                    'http': proxy_url,
                    'https': proxy_url,
                }

                # 设置 SOCKS5 代理时
                if proxy_type.lower() == 'socks5':
                    socks.set_default_proxy(socks.SOCKS5, ip, port, username=user, password=passwd)
                    socket.socket = socks.socksocket  # 重置 socket 为 socks 代理
                else:
                    # 对于 HTTP 代理的场景
                    kwargs['proxies'] = proxies

                # 调用被装饰的函数，传递代理配置
                return func(*args, **kwargs)
            return wrapper
        return decorator





if __name__=='__main__':   
    ip = '192.168.1.65'#把这里替换成你服务器实际的信息即可
    port = 2025
    tocken='LTtx'
    tocken='test'
    
    tx1=txl(ip,port,tocken)
    tx1.start_tx()
    tx1.start_txg('litao')

    #

    def show():
        while True:
            data = tx1.Q.get().split('|')
            print(time.time() - float(data[1]))
            # print(t1-float(data[1]))
    threading.Thread(target = show).start()
   
    # #使用内部代理访问当前ip
    # @txl.proxy('127.0.0.1', 8080)
    # def fetch_data():
    #     response = requests.get('http://cip.cc')
    #     print(response.text)

    # @txl.proxy('127.0.0.1', 2028,user='test2',passwd='2333', proxy_type='socks5')
    # def fetch_socks5_data():
    #     import requests        
    #     response = requests.get('http://cip.cc')
    #     print(response.text)

    # fetch_socks5_data()







from setuptools import setup, find_packages

setup(
    name='LTtx',
    version='7.0.4',
    description='A Python wrapper for socket-based communication',
    long_description=open('README.md', 'r', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    author='txquant',
    author_email='445646258@qq.com',
    url='https://github.com/95ge/LTtx',
    license='MIT',
    packages=find_packages(),  # 自动查找并包含包
    include_package_data=True,  # 包括包内的其他文件
    package_data={
        'tx': ['Config.txt'],  # 指定需要打包的额外文件
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.5',
    install_requires=[
        'pandas_market_calendars',
        'pyzmq',
        'requests',
        'pandas',
    ],
    entry_points={
        'console_scripts': [
            'LTtx_server=tx.LTtx_server:main',  # 定义入口点
        ],
    },
)
